@@ -75,6 +75,8 @@ class DPiFuncGen:
     #    Enter:  boardNumber = DPiFuncGen board number (0 - 3)
     #
     def setBoardNumber(self, boardNumber: int):
+        if (boardNumber < 0) or (boardNumber > 3):
+            boardNumber = 0
         self._slaveAddress = _DPiNETWORK_BASE_ADDRESS + boardNumber
 
 
