@@ -86,7 +86,7 @@ class DPiRobot:
     STATE_MOTORS_DISABLED = 3                   # motors are disabled
     STATE_NOT_HOMED = 4                         # motors enabled but robot not homed (either hasn't homed or homing failed)
     STATE_HOMING = 5                            # robot is running the homing procedure 
-    STATE_STOPPED = 6                           # robot is stopped, waiting for waypoints from the hose
+    STATE_STOPPED = 6                           # robot is stopped, waiting for waypoints from the host
     STATE_PREPARING_TO_MOVE = 7                 # robot has received 1 or more waypoints but hasn't started moving yet
     STATE_MOVING = 8                            # moving to next waypoint
     STATE_WAYPOINT_BUFFER_FULL_FLAG = 0x80      # this bit set in status byte if waypoint buffer is full
@@ -338,6 +338,15 @@ class DPiRobot:
     #    Exit:   True returned on success, else False
     #
     def addWaypoint(self, X: float, Y: float, Z: float, speed: float):
+        #
+        # verify values are in range
+        #
+        if (X < -3275) or (X > 3275) or (Y < -3275) or (Y > 3275) or (Z < -3275) or (Z > 3275):
+            return False
+        
+        if (speed < 0.001) or (speed > 3275):
+            return False
+        
         #
         # loop if waypoint buffer is full
         #
@@ -375,6 +384,15 @@ class DPiRobot:
     #    Exit:   True returned on success, else False
     #
     def addWaypoint_RobotCoords(self, X: float, Y: float, Z: float, speed: float):
+        #
+        # verify values are in range
+        #
+        if (X < -3275) or (X > 3275) or (Y < -3275) or (Y > 3275) or (Z < -3275) or (Z > 3275):
+            return False
+        
+        if (speed < 0.001) or (speed > 3275):
+            return False
+
         #
         # loop if waypoint buffer is full
         #
@@ -481,4 +499,242 @@ class DPiRobot:
         #
         return True
 
+ 
+
+    # ---------------------------------------------------------------------------------
+    #                     Public functions configuring the robot
+    # ---------------------------------------------------------------------------------
+
+    #
+    # set the robot type and initialize it
+    #  Enter:  typeOfRobot = ROBOT_TYPE_ROTATIONAL_DELTA, ROBOT_TYPE_LINEAR_DELTA, ROBOT_TYPE_CARTESIAN...
+    #  Exit:   true returned on success, else false
+    #
+    def setRobotType(self, typeOfRobot: int):
+        dpiNetwork.pushUint8(typeOfRobot)
+        return self.__sendCommand(_CMD_DPiROBOT__SET_ROBOT_TYPE)
+
+
+    # 
+    # for all robot types: set default acceleration
+    #   Enter:  acceleration = steps per second per second
+    #   Exit:   true returned on success, else false
+    # 
+    def setRobotDefaultAcceleration(self, acceleration: float):
+        self.__pushFltPoint0ToInt32(acceleration)
+        return self.__sendCommand(_CMD_DPiROBOT__SET_ROBOT_DEFAULT_ACCELERATION)
+
+
+    # 
+    # for all robot types: set default junction deviation 
+    # The "Junction Deviation" defines how much the motors will slow down when changing 
+    # direction.  Small changes in direction typically need only a small reduction in speed, 
+    # while sharp turns require a much larger reduction in speed.  For example, if the path 
+    # between two waypoints are at a right angle to each other, the combined motion of all 
+    # the motors must slow down significately to make this sharp turn.  Setting the
+    # Junction Deviation constant controls how much the motors will slow down as they 
+    # head into, and out of turns.  Effectively this sets the cornering speed.  But
+    # this does not "round" corners, in that it will always complete all steps to reach 
+    # the exact waypoint coordinate.  The default value is set around 1.0.  The larger
+    # the value, the less it slows down when changing direction.  When moving a large
+    # mass, or with a very ridge system, a smaller number may be needed.  If the number
+    # is too large, it may cause a loss of steps as sharp turns are made.  Units are in 
+    # steps. This value can only be changed when all motion is stopped.
+    # 
+    def setRobotDefaultJunctionDeviation(self, junctionDeviation: float):
+        self.__pushFltPoint0ToInt32(junctionDeviation);
+        return self.__sendCommand(_CMD_DPiROBOT__SET_ROBOT_DEFAULT_JUNCTION_DEVIATION)
+
+
+    # 
+    # for all robot types: set minimum and maximum X
+    #  Enter:  minimumX, maximumX = min and max values of X in millimeters (−3276.8mm to 3276.7mm)
+    #  Exit:   true returned on success, else false
+    # 
+    def setRobotMinMaxX(self, minimumX: float, maximumX: float):
+        self.__pushFltPoint1ToInt16(minimumX)
+        self.__pushFltPoint1ToInt16(maximumX)
+        return self.__sendCommand(_CMD_DPiROBOT__SET_ROBOT_MIN_MAX_X)
+
+
+    # 
+    # for all robot types: set minimum and maximum Y
+    #  Enter:  minimumY, maximumY = min and max values of Y in millimeters (−3276.8mm to 3276.7mm)
+    #  Exit:   true returned on success, else false
+    # 
+    def setRobotMinMaxY(self, minimumY: float, maximumY: float):
+        self.__pushFltPoint1ToInt16(minimumY)
+        self.__pushFltPoint1ToInt16(maximumY)
+        return self.__sendCommand(_CMD_DPiROBOT__SET_ROBOT_MIN_MAX_Y)
+
+
+    # 
+    # for all robot types: set minimum and maximum Z
+    #  Enter:  minimumZ, maximumZ = min and max values of Z in millimeters (−3276.8mm to 3276.7mm)
+    #  Exit:   true returned on success, else false
+    # 
+    def setRobotMinMaxZ(self, minimumZ: float, maximumZ: float):
+        self.__pushFltPoint1ToInt16(minimumZ)
+        self.__pushFltPoint1ToInt16(maximumZ)
+        return self.__sendCommand(_CMD_DPiROBOT__SET_ROBOT_MIN_MAX_Z)
+
+
+    # 
+    # for all robot types: set homing speed
+    #  Enter:  speedDegOrMMPerSec = speed in: deg/sec for rot delta robots, mm/sec for linear delta robots, 
+    #                               mm/sec for cartesian robots (0 to 2000mm/sec)
+    #  Exit:   true returned on success, else false
+    # 
+    def setHomingSpeed(self, speedDegOrMMPerSec: float):
+        self.__pushFltPoint1ToInt16(speedDegOrMMPerSec)
+        return self.__sendCommand(_CMD_DPiROBOT__SET_HOMING_SPEED)
+
+
+    # 
+    # set the homing method type
+    #  Enter:  homingMethod = ROBOT_HOMING_METHOD_LIMIT_SWITCHES, ROBOT_HOMING_METHOD_STALL_DETECTION, ROBOT_HOMING_METHOD_NONE...
+    #  Exit:   true returned on success, else false
+    # 
+    def setHomingMethod(self, homingMethod: int):
+        dpiNetwork.pushUint8(homingMethod)
+        return self.__sendCommand(_CMD_DPiROBOT__SET_HOMING_METHOD)
+
+
+    # 
+    # for all robot types: set maximum distance to move when homing
+    #  Enter:  maxDistanceDegOrMM = maximum distance to move when homing (Degrees or MM)  (0 to 3276.7)
+    #  Exit:   true returned on success, else false
+    # 
+    def setMaxHomingDistanceDegOrMM(self, maxDistanceDegOrMM: float):
+        self.__pushFltPoint1ToInt16(maxDistanceDegOrMM)
+        return self.__sendCommand(_CMD_DPiROBOT__SET_MAX_HOMING_DISTANCE_DEG_OR_MM)
+
+
+    # 
+    #  linear delta robot: set steps/mm
+    #  Enter:  stepsPerMM = number of steps per MM of motion for the arm tower
+    # 
+    def linDelta_SetStepsPerMM(self, stepsPerMM: float):
+        self.__pushFltPoint3ToInt32(stepsPerMM)
+        return self.__sendCommand(_CMD_DPiROBOT__LIN_DELTA_SET_STEPS_PER_MM)
+
+
+    # 
+    # set linear delta robot: arm length
+    #  Enter:  armLengthMM = length in mm of arm
+    #  Exit:   true returned on success, else false
+    # 
+    def linDelta_SetArmLength(self, armLengthMM: float):
+        self.__pushFltPoint3ToInt32(armLengthMM)
+        return self.__sendCommand(_CMD_DPiROBOT__LIN_DELTA_SET_ARM_LENGTH)
+
+
+    # 
+    # set linear delta robot: raduis of the robot's tower and end effector
+    #  Enter:   towerRadiusMM = radius of the tower, from its center to the rotational joints of the arms
+    #           endEffectorRadiusMM = radius of the end effector, from its center to the rotational joints of the lower arm
+    #  Exit:   true returned on success, else false
+    # 
+    def linDelta_SetTowerAndEndEffectorRadius(self, towerRadiusMM: float, endEffectorRadiusMM: float):
+        self.__pushFltPoint3ToInt32(towerRadiusMM);
+        self.__pushFltPoint3ToInt32(endEffectorRadiusMM)
+        return self.__sendCommand(_CMD_DPiROBOT__LIN_DELTA_SET_TOWER_AND_END_EFFECTOR_RADIUS)
+
+
+    # 
+    # set linear delta robot: maximum arm tower position
+    #  Enter:  maximumJointPos = maximum allowed pos of the arm tower
+    #  Exit:   true returned on success, else false
+    # 
+    def linDelta_SetMaxJointPos(self, maximumJointPos: float):
+        self.__pushFltPoint3ToInt32(maximumJointPos)
+        return self.__sendCommand(_CMD_DPiROBOT__LIN_DELTA_SET_MAX_JOINT_POS)
+
+
+
+    # 
+    #  set rotational delta robot: steps/degree
+    #  Enter:  stepsPerDegree = number of steps per degree of motion for the upper arm
+    #  Exit:   true returned on success, else false
+    # 
+    def rotDelta_SetStepsPerDegree(self, stepsPerDegree: float):
+        self.__pushFltPoint3ToInt32(stepsPerDegree)
+        return self.__sendCommand(_CMD_DPiROBOT__ROT_DELTA_SET_STEPS_PER_DEGREE)
+
+
+    # 
+    # set rotational delta robot: arm lengths
+    #  Enter:  upperArmLengthMM = length in mm of upper arm (arm connected to the motor)
+    #          lowerArmLengthMM = lenght in mm of lower arm (arm connected to the end effector)
+    #  Exit:   true returned on success, else false
+    # 
+    def rotDelta_SetArmLengths(self, upperArmLengthMM: float, lowerArmLengthMM: float):
+        self.__pushFltPoint3ToInt32(upperArmLengthMM)
+        self.__pushFltPoint3ToInt32(lowerArmLengthMM)
+        return self.__sendCommand(_CMD_DPiROBOT__ROT_DELTA_SET_ARM_LENGTH)
+
+
+    #
+    # set rotational delta robot: raduis of the robot's base and end effector
+    #  Enter:  baseRadiusMM = radius of the base, from its center to the rotational joints of the upper arm
+    #          endEffectorRadiusMM = radius of the end effector, from its center to the rotational joints of the lower arm
+    #  Exit:   true returned on success, else false
+    #
+    def rotDelta_SetBaseAndEndEffectorRadius(self, baseRadiusMM: float, endEffectorRadiusMM: float):
+        self.__pushFltPoint3ToInt32(baseRadiusMM)
+        self.__pushFltPoint3ToInt32(endEffectorRadiusMM)
+        return self.__sendCommand(_CMD_DPiROBOT__ROT_DELTA_SET_BASE_AND_END_EFFECTOR_RADIUS)
+
+
+    #
+    # set rotational delta robot: motor angle when upper arm is homed
+    #  Enter:  upperArmHomeAngleDegrees = arm angle in degress when arm in home position, such that 0 degrees 
+    #            represents the arm angle when it's in the horizontal plane
+    #  Exit:   true returned on success, else false
+    #
+    def rotDelta_SetUpperArmHomeAngleDegrees(self, upperArmHomeAngleDegrees: float):
+        self.__pushFltPoint3ToInt32(upperArmHomeAngleDegrees)
+        return self.__sendCommand(_CMD_DPiROBOT__ROT_DELTA_SET_UPPER_ARM_HOME_ANGLE_DEGREES)
+
+
+    #
+    # set rotational delta robot: minimum and maximum upper arm angles
+    #  Enter:  minimumJointAngle = minimum allowed angle of the upper arm
+    #          maximumJointAngle = maximum allowed angle of the upper arm
+    #  Exit:   true returned on success, else false
+    #
+    def rotDelta_SetMinMaxJointAngles(self, minimumJointAngle: float, maximumJointAngle: float):
+        self.__pushFltPoint3ToInt32(minimumJointAngle)
+        self.__pushFltPoint3ToInt32(maximumJointAngle)
+        return self.__sendCommand(_CMD_DPiROBOT__ROT_DELTA_SET_MIN_MAX_JOINT_ANGLES)
+
+
+    #
+    # set for all robot types: driver type
+    #  Enter:  dtype = MOTOR_DRIVER_TYPE_DRV8825, MOTOR_DRIVER_TYPE_TMC2130, MOTOR_DRIVER_TYPE_DM542T...
+    #  Exit:   true returned on success, else false
+    #
+    def motorDriver_SetDriverType(self, dtype: int):
+        dpiNetwork.pushUint8(dtype)
+        return self.__sendCommand(_CMD_DPiROBOT__MOTOR_DRIVER_SET_DRIVER_TYPE)
+
+
+    #
+    # set for all robot types: driver microstepping
+    #  Enter:  mstepping = 16, 32, 64, 128
+    #  Exit:   true returned on success, else false
+    #
+    def motorDriver_SetDriverMicrostepping(self, mstepping: int):
+        dpiNetwork.pushUint8(mstepping)
+        return self.__sendCommand(_CMD_DPiROBOT__MOTOR_DRIVER_SET_DRIVER_MICROSTEPPING)
+
+
+    #
+    # set for all robot types: step direction reversing flag
+    #  Enter:  reverseSteppingFlg = true or false
+    #  Exit:   true returned on success, else false
+    #
+    def motorDriver_SetReverseStepDirectionFlag(self, reverseSteppingFlg: bool):
+        dpiNetwork.pushUint8(reverseSteppingFlg)
+        return self.__sendCommand(_CMD_DPiROBOT__MOTOR_DRIVER_SET_REVERSE_DIRECTION_FLAG)
 
